var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx } from "react/jsx-runtime";
import { useEffect, useState } from "react";
import { PassSDK, RedeemAndRevealSDK, SimultaneousRevealSDK, UseCase, } from "@inkerlabs/sdk";
import Context from "./context";
import { WagmiConfig, createClient, chain, configureChains, useNetwork, useSwitchNetwork, } from "wagmi";
import { alchemyProvider } from "wagmi/providers/alchemy";
import { CoinbaseWalletConnector } from "wagmi/connectors/coinbaseWallet";
import { MetaMaskConnector } from "wagmi/connectors/metaMask";
import { WalletConnectConnector } from "wagmi/connectors/walletConnect";
import { CHAIN_NAMESPACES } from "@web3auth/base";
import useSigner from "./useSigner";
import useWebThreeAuth from "./useWebThreeAuth";
export var availableChains = [
    chain.mainnet,
    chain.goerli,
    chain.polygon,
    chain.polygonMumbai,
];
function buildWeb3AuthChainConfig(chain, apiKey) {
    var _a, _b, _c;
    var config = {
        chainNamespace: CHAIN_NAMESPACES.EIP155,
        chainId: chain.id.toString(16),
        rpcTarget: "".concat(chain.rpcUrls["alchemy"], "/").concat(apiKey),
        displayName: chain.name,
        blockExplorer: (_a = chain.blockExplorers) === null || _a === void 0 ? void 0 : _a.default.url,
        ticker: (_b = chain.nativeCurrency) === null || _b === void 0 ? void 0 : _b.symbol,
        tickerName: (_c = chain.nativeCurrency) === null || _c === void 0 ? void 0 : _c.name,
        testnet: chain.testnet,
    };
    window.chainConfig = config;
}
function _Provider(props) {
    var children = props.children;
    var chain = availableChains.find(function (x) { return x.network === props.config.chain; });
    buildWeb3AuthChainConfig(chain, props.config.alchemyApiKey);
    window.whiteLabel = props.whiteLabel;
    var _a = useState({}), context = _a[0], setContext = _a[1];
    var currentWagmiChain = useNetwork().chain;
    var switchNetwork = useSwitchNetwork().switchNetwork;
    var _b = useSigner(), signer = _b.signer, address = _b.address;
    var infos = useWebThreeAuth().infos;
    console.log("target chain: ", chain.id);
    console.log("current wagmi chain: ", currentWagmiChain === null || currentWagmiChain === void 0 ? void 0 : currentWagmiChain.id);
    useEffect(function () {
        if (signer) {
            var sdk = void 0;
            switch (props.useCase) {
                case UseCase.PASS:
                    sdk = new PassSDK(props.config, signer);
                    break;
                case UseCase.RR:
                    sdk = new RedeemAndRevealSDK(props.config, signer);
                    break;
                case UseCase.SR:
                    sdk = new SimultaneousRevealSDK(props.config, signer);
                    break;
            }
            setContext({
                sdk: sdk,
                error: undefined,
            });
        }
        else {
            var sdk = void 0;
            switch (props.useCase) {
                case UseCase.PASS:
                    sdk = new PassSDK(props.config);
                    break;
                case UseCase.RR:
                    sdk = new RedeemAndRevealSDK(props.config);
                    break;
                case UseCase.SR:
                    sdk = new SimultaneousRevealSDK(props.config);
                    break;
            }
            setContext({
                sdk: sdk,
            });
        }
    }, [props.config]);
    useEffect(function () {
        var _a;
        // @ts-ignore
        (_a = context.sdk) === null || _a === void 0 ? void 0 : _a.updateSigner(address, signer, infos === null || infos === void 0 ? void 0 : infos.idToken);
    }, [address, signer, infos]);
    useEffect(function () {
        if (currentWagmiChain && chain && currentWagmiChain.id != chain.id) {
            switchNetwork && switchNetwork(chain.id);
        }
    }, [chain, currentWagmiChain, switchNetwork]);
    if (!context.sdk) {
        return null;
    }
    return _jsx(Context.Provider, __assign({ value: context }, { children: children }));
}
export default function Provider(_a) {
    var children = _a.children, props = __rest(_a, ["children"]);
    var _b = configureChains(availableChains, [alchemyProvider({ apiKey: props.config.alchemyApiKey })]), chains = _b.chains, provider = _b.provider, webSocketProvider = _b.webSocketProvider;
    // Set up client
    var client = createClient({
        autoConnect: true,
        connectors: [
            new MetaMaskConnector({ chains: chains }),
            new CoinbaseWalletConnector({
                chains: chains,
                options: {
                    appName: "Inker Labs",
                },
            }),
            new WalletConnectConnector({
                chains: chains,
                options: {
                    qrcode: true,
                },
            }),
        ],
        provider: provider,
        webSocketProvider: webSocketProvider,
    });
    return (_jsx(WagmiConfig, __assign({ client: client }, { children: _jsx(_Provider, __assign({}, props, { children: children })) })));
}
